#ifndef _SIDEFS_H_
#define _SIDEFS_H_

#pragma once 

/*===========================================================================

             SI SDK COMMON DATA STRUCTURES

DESCRIPTION
  This file contains the data structures for Silicon Imaging USB camera SDK.

REFERENCES
  USB MegaCamera API Manual

Copyright (c) 2003 by Silicon Imaging, Incorporated. All Rights Reserved.
===========================================================================*/

#if defined( _WIN32 ) || defined ( _WIN64 )
  #define __STDCALL  __stdcall
  #define __CDECL    __cdecl
  #define __INT64    __int64
  #define __UINT64    unsigned __int64
#else
  #define __STDCALL
  #define __CDECL
  #define __INT64    long long
  #define __UINT64    unsigned long long
#endif


#define SI_MAX_PATH						1024		
#define SI_MAX_INIT_VR				32
#define SI_MAX_IPS						16

#ifdef SIDLL_EXPORTS
	#define SIAPI __declspec(dllexport)
#else
	#define SIAPI __declspec(dllimport)
#endif


typedef unsigned char   SI8u;
typedef unsigned short  SI16u;
typedef unsigned int    SI32u;
typedef signed char			SI8s;
typedef signed short		SI16s;
typedef signed int			SI32s;
typedef float						SI32f;
typedef __INT64					SI64s;
typedef __UINT64				SI64u;
typedef double					SI64f;

typedef unsigned char		uint8;
typedef char						int8;
typedef signed short		int16;
typedef unsigned short	uint16;
typedef unsigned int		uint32;
typedef unsigned int		int32;


enum SIResult
{
	SR_OK											= 0,			// No error
	SR_Error									=	-1,			// Generic, unspecifed error
	SR_NotSupported						= -2,
	SR_Abort									= 1,
	SR_ImageDataOrderErr			=	-20,
	SR_ImageDataDepthErr			= -21,
	SR_ImageChannelNumberErr	= -22,
	SR_ImageBufferErr					= -23,
	SR_ImageFormatErr					= -24,
	SR_ImageBppErr						= -25,
	SR_FileIOErr							= -51,
	SR_ReadConfigFileErr			= -52,

	
	SR_CameraIDErr						= -500,
	SR_CameraOpenErr					= -501,
	SR_CameraConfigErr				= -502,
	SR_CameraChangeBppErr			= -503,
	SR_CameraInitErr					= -504,
	SR_CameraModelErr					= -505,
	SR_CameraSnapFailure			= -506,
	SR_CameraSnapDataErr			= -507,
	SR_CameraSnapTimeout			= -508,

	SR_CameraInUse						= 500,
};

typedef struct tagSIPoint
{
	int iX;
	int iY;
}SIPoint;

typedef struct tagSIRegion
{
	int iX, iY, iWidth, iHeight;
}SIRegion;

typedef struct tagSISize
{
	int iWidth, iHeight;
}SISize;

enum SIColorSpace
{
	SICS_Mono			= 0,
	SICS_RGB			= 1,
	SICS_CbYCr422	= 2,
};

enum SIColorMode
{
	SICM_Auto			= 0,
	SICM_Mono			= 1,
	SICM_Color		= 2,	
};

enum SIWhiteBalanceAlgorithm
{
	SIWBPreset					= 0,	
	SIWBGrayWorld				= 1,
	SIWBWhitePatch			= 2,	
};

enum SIBayerPhase
{
	SIBayerPhaseBG = 0,
	SIBayerPhaseGB = 1,
	SIBayerPhaseRG = 2,
	SIBayerPhaseGR = 3
};

enum SIBayerInterpAlgorithm
{
	SIBayerInterpNNP			= 0,
	SIBayerInterpBilinear = 1,
};

enum SIDPCAlgorithm
{
	SIDPC_NULL							= 0,	
	SIDPC_CSSAD							= 1,
};

enum SIBitReductionAlgorithm
{
	SIBRALGO_Truncate		= 0,
};

enum SICameraModel
{
	// Note: use odd number for mono and even number for color.
	SIGenericCam	= 0,
	SI1280M				= 1,
	SI1280C				= 2,  
	SI3170M				= 3,
	SI3170C				= 4,
	SI2100M				= 5,
	SI2100C				= 6,
	SI640M				= 7,
	SI640C				= 8,
	SI1300M				= 9,
	SI1300C				= 10,
	SI3300M				= 11,
	SI3300C				= 12,
	SI6600M				= 13,
	SI6600C				= 14,
	SI5100M				= 15,
	SI5100C				= 16
};

typedef enum tagSIGainSelection
{
	SISelectIndividualGains = 0,
	SISelectGlobalGain = 1	
}SIGainSelection;

typedef enum tagSICAMIF_SizingMode
{
	SICAMIF_SM_Unknown      = -1,
	SICAMIF_SM_Windowing    = 0,
	SICAMIF_SM_Subsampling  = 1,
	SICAMIF_SM_Zooming      = 2,
}SICAMIF_SizingMode;

/*
 * SIU_CAMIFFormat
 *	Defines camera sensor read-out interface format.
 */
typedef struct tagSIU_CAMIFFormat
{
	int iWidth;
	int iHeight;
	int iBpp;
	int iSubSampleMode;
	SICAMIF_SizingMode SizingMode;
	SIPoint Origin;
	SIColorSpace ColorSpace;
}SIU_CAMIFFormat;

typedef enum tagSIU_SnapMode
{
	SIUSM_SnapLoop = 0,
	SIUSM_LiveDriver = 1,
}SIU_SnapMode;

/*
 * SIU_CameraConfig
 *	Defines SI USB MegaCamera camera configuration.
 */

typedef struct tagSIU_CameraConfig
{	
	SIU_CAMIFFormat CAMIFFormat;									// Camera interface format
	int		iClockIndex;														// Clock table index	
	int		iGlobalGain;                            // Global gain table index
	int		iRGGBGain[4];                           // RGGB gain table indices
	SIGainSelection		GainSelection;							// effective only for cameras with mutually exclusive gain settings
	int		iBlackOffset;
	int		iTBRange;																// (1280 only)
	int		iExposureValue;	
	int		iFrameTime;
	int		iColumnOffset;
	int		iVerticalBlanking;
	int		iHorizontalBlanking;	
	int		iAutoMode;                       // SI2100 and SI5100
	int		iAECBrightnessTarget;				// 2100 only
	int		iAECBrightnessBracket;			// 2100 only	
	int		iFastAECThreshold[2];        // SI5100 only
	int		iArmTrigger;
	int		iLiveModeDriverDelay;
	int		iShutterMode;	
	int		iGammaMode;
}SIU_CameraConfig;

/*
 * SIU_ImagingProfile
 *	Defines video front end (VFE) imaging profiles.
 */
typedef struct tagSIU_ImagingProfile
{
	char	szDescription[64];
	float fSaturationMatrix[3][3];
	float fWhiteBalance[3];
	float fGlobalGain;
	float fRGBGain[3];
	float fSaturationFactor;
	int		iGlobalOffset;
	SIBayerPhase							BayerPhase;
	SIBayerInterpAlgorithm		BayerInterpAlgo;
	SIWhiteBalanceAlgorithm		WhiteBalanceAlgo;	
}SIU_ImagingProfile;

/*
 * SIU_ImagingConfig
 *	Defines video front end (VFE) imaging context
 */
typedef struct tagSIU_ImagingConfig
{
	int		iDPCEna;
	int		iMaxDPNum;
	SIDPCAlgorithm DPCAlgo;
	int		iBlackCorrection;
}SIU_ImagingConfig;

typedef struct tagSIU_VendorRequest
{
	int iVRCode;
	int iIndex;
	int iValue;
	int iDirection;
	int iBufferSize;
	unsigned char Buffer[64];
}SIU_VendorRequest;

typedef struct tagSIU_RunStatus
{	
	SIColorMode ColorMode;
	SIU_SnapMode SnapMode;
	int iSnapDelay;
	int iVerbose;
}SIU_RunStatus;

typedef struct tagSIUCConfig
{
	SIU_CameraConfig		CameraConfig;
	SIU_ImagingConfig		ImagingConfig;
	int iActiveImagingProfile;
	int iInitVRUsed;
	int iTotalImagingProfile;
	SIU_RunStatus				RunStatus;
	SIU_ImagingProfile	ImagingProfile[SI_MAX_IPS];
	SIU_VendorRequest		InitVR[SI_MAX_INIT_VR];
}SIU_Config;

typedef struct tagSIMonoImage
{
	int iWidth;
	int iHeight;
	int iPitch;
	int iBpp;
	union
	{
		uint8		* pData8u;
		uint16	* pData16u;
		int16		* pData16s;
	}Data;
}SIMonoImage;

typedef enum
{
	SICO_Pixel	= 0,
	SICO_Planar	= 1,
}SIColorOrder;

typedef struct tagSIColorImage
{
	int iWidth;
	int iHeight;
	int iPitch;
	int iBpp;
	union
	{
		uint8		* pData8u;
		uint16	* pData16u;
		int16		* pData16s;
		uint8		* pPlane8u[3];
		uint16	* pPlane16u[3];
		int16		* pPlane16s[3];
	}Data;
	SIColorOrder ColorOrder;
}SIColorImage;

typedef struct tagSIPackedDIB
{
	BITMAPINFOHEADER	BmpInfo;
	RGBQUAD						ColorTable[256];
	PBYTE							pDIBBits;
}SIPackedDIB;

typedef struct tagSIU_VFEConfig
{
	SIU_CAMIFFormat			CAMIFFormat;
	SIU_RunStatus				RunStatus;
	SIU_ImagingProfile	ImagingProfile;
	SIU_ImagingConfig		ImagingConfig;
}SIU_VFEConfig;

typedef struct tagSIU_VFEContext
{
	SIMonoImage				BayerImage;
	SIColorImage			RGBImageP3;
	SIColorImage			RGBImageC3;
	SIPackedDIB				PackedDIB;
}SIU_VFEContext;

#endif